@@ -30,4 +30,4 @@ class CallbackList(object):
         for cb in self.callbacks:
             if cb.name == callback_name:
                 return cb
-        return cb
+        return None
